﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Xml.Serialization;
using MyFriends.Business;

namespace MyFriends.Model
{
    [Serializable()]
    public class Person
    {
        public Person()
        {
            Birthday = new DateTime(1980, 1, 1);
        }

        public string FirstName { get; set; }
        public string MiddleName { get; set; }
        public string LastName { get; set; }
        //public int Age { get; set; } // Configure this later to be now - Birthday converted to years.
        public DateTime Birthday { get; set; }
        public FriendType Type { get; set; }

        public bool IsEmpty
        {
            get
            {
                return String.IsNullOrWhiteSpace(FirstName)
                    && String.IsNullOrWhiteSpace(MiddleName)
                    && String.IsNullOrWhiteSpace(LastName);
            }
        }

        public enum FriendType
        {
            Friend,
            Family,
            Aquaintence,
            Colleague
        }
    }

    [Serializable()]
    [XmlRoot("MyFriends")]
    public class PersonCollection : ObservableCollection<Person>
    {
        #region Static
        public static PersonCollection Instance
        {
            get
            {
                if (_Instance == null)
                {
                    if (_XmlPath == null || !File.Exists(_XmlPath))
                        _Instance = new PersonCollection();
                    else
                        _Instance = Serializer.DeserializeFromXML<PersonCollection>(_XmlPath);
                }
                return _Instance;
            }
            set { _Instance = value; }
        } private static PersonCollection _Instance;

        public static string XmlPath
        {
            get { return _XmlPath; }
            set { _XmlPath = value; }
        } private static string _XmlPath;

        #endregion

        public PersonCollection()
        {
        }

        public ObservableCollection<String> Names
        {
            get
            {
                var names = from Person in this select Person.LastName + ", " + Person.FirstName;
                return new ObservableCollection<String>(names);
            }
        }

        public void SerializeToXml(string inXmlPath = null)
        {
            if (!string.IsNullOrWhiteSpace(inXmlPath))
                _XmlPath = inXmlPath;
            else if (!string.IsNullOrWhiteSpace(_XmlPath))
                Serializer.SerializeToXML<PersonCollection>(this, _XmlPath);
            else
                throw new ArgumentNullException("inXmlPath cannot be null if XmlPath is not set.");
        }
    }
}