﻿using System.Windows.Input;
using MVVM;
using MyFriends.Model;

namespace MyFriends.ViewModel
{
    class FriendListViewModel : ViewModelBase
    {
        #region Member Variables
        #endregion

        #region Constructors
        /// <summary>
        /// The default constructor
        /// </summary>
        public FriendListViewModel()
        {
            PersonCollection.XmlPath = @"Data\MyFriends.xml";
        }
        #endregion

        #region Properties
        public PersonCollection People
        {
            get { return PersonCollection.Instance; }
            set { PersonCollection.Instance = value; NotifyPropertyChanged("People"); }
        }

        public Person SelectedFriend
        {
            get { return _SelectedFriend; }
            set
            {
                _SelectedFriend = value;
                NotifyPropertyChanged("SelectedFriend");
            }
        } public Person _SelectedFriend;

        #region ICommands
        public ICommand ExitCommand
        {
            get
            {
                if (_ExitCommand == null)
                    _ExitCommand = new RelayCommand(param => Exit());
                return _ExitCommand;
            }
        } private RelayCommand _ExitCommand;
        
        public ICommand SaveCommand
        {
            get
            {
                if (_SaveCommand == null)
                    _SaveCommand = new RelayCommand(param => SaveFriends());
                return _SaveCommand;
            }
        } private RelayCommand _SaveCommand;

        public ICommand NewFriendCommand
        {
            get
            {
                if (_NewFriendCommand == null)
                    _NewFriendCommand = new RelayCommand(param => AddFriend());
                return _NewFriendCommand;
            }
        } private RelayCommand _NewFriendCommand;
        
        public ICommand DeleteIfEmptyCommand
        {
            get
            {
                if (_DeleteIfEmptyCommand == null)
                    _DeleteIfEmptyCommand = new RelayCommand(param => DeleteIfEmpty());
                return _DeleteIfEmptyCommand;
            }
        } private RelayCommand _DeleteIfEmptyCommand;

        public ICommand DeleteFriendCommand
        {
            get
            {
                if (_DeleteFriendCommand == null)
                    _DeleteFriendCommand = new RelayCommand(param => DeleteFriend());
                return _DeleteFriendCommand;
            }
        } private RelayCommand _DeleteFriendCommand;
        #endregion
        #endregion

        #region Functions
        private void Exit()
        {
            SaveFriends();
            App.Current.Shutdown(0);
        }

        private void SaveFriends()
        {
            People.SerializeToXml();
        }

        private void AddFriend()
        {
            Person newFriend = new Person();
            People.Add(newFriend);
            SelectedFriend = newFriend;
        }

        private void DeleteIfEmpty()
        {
            if (SelectedFriend.IsEmpty)
                DeleteFriend();
        }

        private void DeleteFriend()
        {
            Person friendToDelete = SelectedFriend;
            SelectedFriend = null;
            People.Remove(friendToDelete);
        }
        #endregion

        #region Enums
        #endregion
    }
}
